#pragma once
#include "olcPixelGameEngine.h"
#include "Unit.h"

namespace util{
	float random(float range);
	void ApplyMatrixEffect(PixelGameEngine*pge,Renderable&r,Renderable&originalImg,std::unique_ptr<Renderable>&matrixImg);
	bool CanAfford(Resources&resources,std::vector<Memory>&unitCosts);
	int GetHealthCost(std::vector<Memory>&unitCosts);
	int GetAtkSpdCost(std::vector<Memory>&unitCosts);
	int GetMoveSpdCost(std::vector<Memory>&unitCosts);
	int GetRangeCost(std::vector<Memory>&unitCosts);
	int GetProcedureCost(std::vector<Memory>&unitCosts);
}